from epure import epure
from ..tg_environment.t_user import TUser
from ..finance.currency import Currency
from ..tg_environment.t_message import TMessage
from .offer_state import OfferState
from .offer_type import OfferType
from uuid import UUID, uuid4
from datetime import datetime
from ..tg_environment.telegram_entity import TelegramEntity

# @epure()
class TOffer:
    from_user: TUser
    to_user: TUser
    offer_type: str
    offer_state: str
    subject: TelegramEntity
    currency: Currency
    price: float
    datetime: datetime
    duration: int
    offer_message: TMessage
    bidding_id: UUID

    def __init__(self,    
            from_user: TUser,
            to_user: TUser,
            offer_type: str,
            offer_state: str,
            subject: TMessage,
            currency: Currency,
            price: float,            
            offer_message: TMessage,
            duration: int=None,
            bidding_id: UUID=None
            ):
        self.from_user = from_user
        self.to_user = to_user
        self.offer_type = offer_type
        self.offer_state = offer_state
        self.subject = subject
        self.currency = currency
        self.price = price
        self.offer_message = offer_message
        self.duration = duration
        self.bidding_id = bidding_id
        self.datetime = datetime.now()

        if not self.bidding_id:
            self.bidding_id = uuid4()


from epure import epure
from .taalc_transaction import TaalcTransaction
from .taalc_nft import TaalcNft
from .taalc_nft_token import TaalcNftToken
# from .currency_transaction import CurrencyTransaction
# from epure import Elist


class NftTransaction(TaalcTransaction):    
    taalc_nft_token: TaalcNftToken
    amount: int    

    def __init__(self, sent_from, sent_to, taalc_nft_token, transaction_batch=None, amount=1):
        super().__init__()

        self.sent_from = sent_from
        self.sent_to = sent_to
        self.taalc_nft_token = taalc_nft_token
        self.transaction_batch = transaction_batch
        self.amount = amount


from __future__ import annotations
from typing import TYPE_CHECKING
# from ..finance.taalc_nft import TaalcNft
# from ..finance.taalc_nft_token import TaalcNftToken

from .t_user import TUser
from .t_chat import TChat
from .. import bidding
from .. import finance
if TYPE_CHECKING:
    from ..bidding.message_offer import MessageOffer
    from ..finance.message_nft_token import MessageNftToken
from epure import epure
from aiogram import types
from .telegram_entity import TelegramEntity


@epure()
class TMessage(TelegramEntity):
    # owner: TUser
    creator: TUser
    # if TYPE_CHECKING:
    taalc_offer: 'bidding.message_offer.MessageOffer'
    message_nft_token: 'finance.message_nft_token.MessageNftToken'
    taalc_chat: TChat
    tg_chat_id: int
    text: str

    @property
    def owner(self):
        if not (self.message_nft_token and  hasattr(self.message_nft_token, 'data_id')):
           return self.creator
        from ..finance.message_transaction import MessageTransaction
        nft_token_id = self.message_nft_token.data_id
        transactions = MessageTransaction.resource.read(taalc_nft_token = nft_token_id)
        if not transactions:
            return self.creator
        transactions = sorted(transactions, key=lambda tr: tr.transaction_time)
        last_tr = transactions[-1]
        return last_tr.sent_to
        

    def __init__(self, message: types.Message=None):
        
        if not message:
            return
        self.taalc_offer = None
        self.message_nft_token = None
        self.creator = TUser.user_by_tg_user(message.from_user)
        # self.owner = TUser.user_by_tg_user(message.from_user)
        self.tg_chat_id=message.chat.id
        self.telegram_id=message.message_id
        self.text = message.text

        taalc_chat = TChat.resource.read(telegram_id=self.tg_chat_id)
        if taalc_chat:
            taalc_chat = taalc_chat[0]
        else:
            taalc_chat = TChat()
            taalc_chat.telegram_id = message.chat.id
            taalc_chat.shifted_id = message.chat.shifted_id
        self.taalc_chat = taalc_chat

    @classmethod
    def get_t_message(cls, message: types.Message) -> TMessage:
        res = cls.resource.read(tg_chat_id=message.chat.id, \
                                telegram_id=message.message_id)
        if not res:
            res = cls(message)
            res.save()
        else:
            res = res[0]
        
        return res
    
    def get_url(self):
        res = f"https://t.me/c/{self.taalc_chat.shifted_id}/{self.telegram_id}"
        return res
    

        


# from __future__ import annotations
# from typing import Optional, TYPE_CHECKING
from epure import epure
from .currency import Currency
from ..tg_environment.t_member import TMember
from ..tg_environment.t_user import TUser
# if TYPE_CHECKING:
from .taalc_transaction import TaalcTransaction
from datetime import datetime
from .transaction_batch import TransactionBatch

@epure()
class CurrencyTransaction(TaalcTransaction):

    currency: Currency
    amount: float


    def __init__(self, sent_from: TUser, sent_to: TUser,\
                  currency: Currency, amount: float, batch: TransactionBatch=None):

        self.sent_from = sent_from
        self.sent_to = sent_to
        self.currency = currency
        self.amount = amount
        self.transaction_time = datetime.now()

        if batch:
            self.transaction_batch = batch


from .nft_transaction import NftTransaction
from epure import epure
from .message_nft_token import MessageNftToken

@epure()
class MessageTransaction(NftTransaction):
    taalc_nft_token: MessageNftToken 